{
Xiangqi 1.0
===========

Description  : A Chinese chess (Xiang Qi) component for Delphi (version 3 and above)
Release date : 10/09/1998
Author       : Axel Fontaine
E-Mail       : axel-fontaine@usa.net
CopyRight    : Axel Fontaine 1998
Cost         : free, or maybe an e-mail to tell me what you think about TXiangqi/NetXiangqi
Version      : 1.0 (initial release)
Recommended config:
  Delphi 3 or higher
  800x600 or higher
  16, 24 or 32 bit color


Legal Issues
============

This component is my creation and the demos are distributed under the GNU GPL.
If you make changes to NetXiangqi, please send them back to me.
The cartoon cliparts are from Corel Mega Gallery (http://www.corel.com).
The TCP/IP component is part of ICS from Franois PIETTE (http://www.rtfm.be/fpiette/indexuk.htm).
The GoToWeb component is from Sylvain CRESTO (http://www.mygale.org/~cresto).
This component and the examples are provided "as-is" with no warranty of any kind.
I may not be held liable for any damage caused by this component or the examples.
You are not allowed to modify or remove the copyright notices in any of the source files.
You cannot claim you wrote these files.
ICS and GoToWeb files distributed for compiling purposes only. 
I stronly recommend you get the full packages at the indicated URLs.
}


unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Xiangqi, Menus;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    Game1: TMenuItem;
    New1: TMenuItem;
    Exit1: TMenuItem;
    Memo1: TMemo;
    N1: TMenuItem;
    Pieces1: TMenuItem;
    Cartoon1: TMenuItem;
    Chinese1: TMenuItem;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    PaintBox1: TPaintBox;
    PaintBox2: TPaintBox;
    Xiangqi1: TXiangqi;
    Move1: TMenuItem;
    Previous1: TMenuItem;
    Next1: TMenuItem;
    N2: TMenuItem;
    First1: TMenuItem;
    Last1: TMenuItem;
    procedure New1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Xiangqi1Check(Sender: TObject; Player: TPlayerColor);
    procedure Xiangqi1Mate(Sender: TObject; Player: TPlayerColor;
      MateType: TMateType);
    procedure Xiangqi1Move(Sender: TObject; StartPos, EndPos: TPoint);
    procedure Chinese1Click(Sender: TObject);
    procedure Cartoon1Click(Sender: TObject);
    procedure Xiangqi1Capture(Sender: TObject; StartPos, EndPos: TPoint;
      CapturedPiece: PPieceInfo);
    procedure PaintBox1Paint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PaintBox2Paint(Sender: TObject);
    procedure Xiangqi1PieceSetChange(Sender: TObject;
      NewPieceSet: TPieceSet);
    procedure Previous1Click(Sender: TObject);
    procedure Next1Click(Sender: TObject);
    procedure First1Click(Sender: TObject);
    procedure Last1Click(Sender: TObject);
    procedure Xiangqi1NextMove(Sender: TObject);
    procedure Xiangqi1PreviousMove(Sender: TObject);
    procedure Xiangqi1ReKill(Sender: TObject; Piece: PPieceInfo);
    procedure Xiangqi1Resurrect(Sender: TObject; Piece: PPieceInfo);
  private
    { Private declarations }
  public
    CapturedPieces1, CapturedPieces2: TList;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.New1Click(Sender: TObject);
begin
  Memo1.Lines.Clear;
  CapturedPieces1.Clear;
  CapturedPieces2.Clear;
  PaintBox1.Refresh;
  PaintBox2.Refresh;
  Xiangqi1.NewGame;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.Xiangqi1Check(Sender: TObject; Player: TPlayerColor);
var
  Str: String;

begin
  If Player = pcRed Then Str := 'Red' Else Str := 'Green';
  Str := Str + ' is in check !';
  Memo1.Lines.Add(Str);
end;

procedure TForm1.Xiangqi1Mate(Sender: TObject; Player: TPlayerColor;
  MateType: TMateType);
var
  Str: String;

begin
  If Player = pcRed Then Str := 'Red' Else Str := 'Green';
  Str := Str + ' loses by ';
  If MateType = mtCheckMate Then Str := Str + 'checkmate' Else Str := Str + 'stalemate';
  Str := Str + ' !';
  Memo1.Lines.Add(Str);
end;

procedure TForm1.Xiangqi1Move(Sender: TObject; StartPos, EndPos: TPoint);
var
  Str: String;

begin
  If Xiangqi1.Turn = pcRed Then Str := 'Red' Else Str := 'Green';
  Str := Str + ' performs move '+IntToStr(Xiangqi1.CurrentMove)+
               ' from ('+IntToStr(StartPos.X)+','+IntToStr(StartPos.Y)+
               ') to ('+IntToStr(EndPos.X)+','+IntToStr(EndPos.Y)+').';
  Memo1.Lines.Add(Str);
end;

procedure TForm1.Chinese1Click(Sender: TObject);
begin
  Chinese1.Checked := True;
  Xiangqi1.PieceSet := psChinese;
end;

procedure TForm1.Cartoon1Click(Sender: TObject);
begin
  Cartoon1.Checked := True;
  Xiangqi1.PieceSet := psCartoon;
end;

procedure TForm1.Xiangqi1Capture(Sender: TObject; StartPos, EndPos: TPoint;
  CapturedPiece: PPieceInfo);
var
  List: TList;
  PaintBox: TPaintBox;
  Str: String;

begin
  If Xiangqi1.Turn = pcRed Then
    Begin
      PaintBox := PaintBox1;
      List := CapturedPieces1;
      Str := 'Red';
    End
  Else
    Begin
      PaintBox := PaintBox2;
      List := CapturedPieces2;
      Str := 'Green';
    End;
  Str := Str + ' captures at ('+IntToStr(EndPos.X)+','+IntToStr(EndPos.Y)+
                     ') from ('+IntToStr(StartPos.X)+','+IntToStr(StartPos.Y)+').';
  Memo1.Lines.Add(Str);
  List.Add(CapturedPiece);
  PaintBox.Refresh;
end;

procedure TForm1.PaintBox1Paint(Sender: TObject);
var
  I: Integer;

begin
  For I := 0 To Pred(CapturedPieces1.Count) Do
    Xiangqi1.DrawPiece(PaintBox1.Canvas,
                       Point((I mod 5)* PieceWidth,(I div 5)* PieceWidth),
                       CapturedPieces1[I]);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  CapturedPieces1 := TList.Create;
  CapturedPieces2 := TList.Create;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  CapturedPieces2.Free;
  CapturedPieces1.Free;
end;

procedure TForm1.PaintBox2Paint(Sender: TObject);
var
  I: Integer;

begin
  For I := 0 To Pred(CapturedPieces2.Count) Do
    Xiangqi1.DrawPiece(PaintBox2.Canvas,
                       Point((I mod 5)* PieceWidth,(I div 5)* PieceWidth),
                       CapturedPieces2[I]);
end;

procedure TForm1.Xiangqi1PieceSetChange(Sender: TObject;
  NewPieceSet: TPieceSet);
begin
  PaintBox1.Refresh;
  PaintBox2.Refresh;
end;

procedure TForm1.Previous1Click(Sender: TObject);
begin
  Xiangqi1.PreviousMove;
end;

procedure TForm1.Next1Click(Sender: TObject);
begin
  Xiangqi1.NextMove;
end;

procedure TForm1.First1Click(Sender: TObject);
begin
  Xiangqi1.GoToMove(0);
end;

procedure TForm1.Last1Click(Sender: TObject);
begin
  Xiangqi1.GoToMove(Xiangqi1.LastMove);
end;

procedure TForm1.Xiangqi1NextMove(Sender: TObject);
begin
  Memo1.Lines.Add('Next move.');
end;

procedure TForm1.Xiangqi1PreviousMove(Sender: TObject);
begin
  Memo1.Lines.Add('Previous move.');
end;

procedure TForm1.Xiangqi1ReKill(Sender: TObject; Piece: PPieceInfo);
begin
  If Xiangqi1.Turn = pcRed Then
    Begin
      CapturedPieces1.Add(Piece);
      PaintBox1.Refresh;
    End
  Else
    Begin
      CapturedPieces2.Add(Piece);
      PaintBox2.Refresh;
    End;
  Memo1.Lines.Add('And once more the piece returned to ashes ...');
end;

procedure TForm1.Xiangqi1Resurrect(Sender: TObject; Piece: PPieceInfo);
begin
  If Xiangqi1.Turn = pcRed Then
    Begin
      CapturedPieces1.Delete(Pred(CapturedPieces1.Count));
      PaintBox1.Refresh;
    End
  Else
    Begin
      CapturedPieces2.Delete(Pred(CapturedPieces2.Count));
      PaintBox2.Refresh;
    End;
  Memo1.Lines.Add('And the piece came back to life ...');
end;

end.
