{
Xiangqi 1.0
==============

Description  : A chinese chess (Xiang Qi) component for Delphi
Release date : September 1st 1998
Author       : Axel Fontaine
E-Mail       : axel-fontaine@usa.net
CopyRight    : Axel Fontaine 1998
Cost         : free, or maybe an e-mail to tell me what you think about it
Recommended config:
  Delphi 3 or higher
  800x600 or higher
  16, 24 or 32 bit color


Legal Issues
============

This component is my creation. If you modify the source, you have to make the
changes available freely to everyone and I would appreciate it if you'd send me
copy. This component cannot be sold. Not alone, not in a component suite, not in
a modified version, never. If you plan to use this component in a commercial
application, you have to ask my permission before doing so.
The demos are distributed under the GNU GPL.
If you make changes to NetXiangqi, please send them back to me.
The cartoon cliparts are from Corel Mega Gallery (http://www.corel.com).
The TCP/IP component is part of ICS from Franois PIETTE
(http://www.rtfm.be/fpiette/indexuk.htm).
The GoToWeb component is from Sylvain CRESTO
(http://www.mygale.org/~cresto).
This component and the examples are provided "as-is" with no warranty of any
kind. I may not be held liable for any damage caused by this component or the
examples. You are not allowed to modify or remove the copyright notices in any
of the source files. You cannot claim you wrote these files.
ICS and GoToWeb files distributed for compiling purposes only.
I stronly recommend you get the full packages at the indicated URLs.
}


unit Xiangqi;

{$B-} //short-circuit boolean eval

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls;

const
  MaxXPieces = 9;    //Board width in cells
  MaxYPieces = 10;   //Board height in cells
  MaxPieceBitmaps = 14;
  PieceWidth = 40;
  PieceHeight = 40;

type
  TPlayerColor = (pcRed,pcGreen);

  TPlayerSet = Set of TPlayerColor;

  TPieceType = (ptPawn,ptCannon,ptElephant,ptKnight,ptRook,ptGuard,ptKing);

  TPieceInfo = record
    Color: TPlayerColor;
    PieceType: TPieceType;
    ImageIndex: Integer;
  end;

  PPieceInfo = ^TPieceInfo;

  TMove = record
    StartPos: TPoint;
    EndPos: TPoint;
    Piece: PPieceInfo;
    CapturedPiece: PPieceInfo;
  End;

  PMove = ^TMove;

  TPieceSet = (psCartoon,psChinese);

  TGameBoard = array[0..Pred(MaxXPieces),0..Pred(MaxYPieces)] of PPieceInfo;

  TMateType = (mtCheckMate,mtStaleMate);

  TOnCheckEvent = procedure(Sender: TObject; Player: TPlayerColor) of object;
  TOnMateEvent = procedure(Sender: TObject; Player: TPlayerColor; MateType: TMateType) of object;

  TOnCaptureEvent =
    procedure(Sender: TObject; StartPos, EndPos: TPoint; CapturedPiece: PPieceInfo) of object;
  TOnMoveEvent = procedure(Sender: TObject; StartPos, EndPos: TPoint) of object;

  TVoodooEvent = procedure(Sender: TObject; Piece: PPieceInfo) of object;

  TOnPieceSetChangeEvent = procedure(Sender: TObject; NewPieceSet: TPieceSet) of object;

  TXiangqi = class(TCustomControl)
  private
    FBoardBitmap: TBitmap;
    FSrcPosClick: TPoint;
    FDestPosClick: TPoint;
    FGameBoard: TGameBoard;
    FInCheck: TPlayerSet;
    FMove: Cardinal;
    FMoveList: TList;
    FPieceBitmaps: TImageList;
    FPieceSet: TPieceSet;
    FPlaying: Boolean;
    FTurn: TPlayerColor;
    FRedKingPos: TPoint;
    FGreenKingPos: TPoint;
    FOnCheck: TOnCheckEvent;
    FOnMate: TOnMateEvent;
    FOnCapture: TOnCaptureEvent;
    FOnMove: TOnMoveEvent;
    FOnPreviousMove: TNotifyEvent;
    FOnNextMove: TNotifyEvent;
    FOnResurrect: TVoodooEvent;
    FOnReKill: TVoodooEvent;
    FOnPieceSetChange: TOnPieceSetChangeEvent;
    procedure ClearBoardPos(Pos:TPoint);
    procedure DrawPieces;
    procedure OrganizeBitmaps;

    //actual Xiangqi
    function HasCrossedRiver(Player: TPlayerColor; Pos: TPoint):Boolean;
    function HasReachedEnd(Player: TPlayerColor; Pos: TPoint):Boolean;
    function IsInCastle(Player: TPlayerColor; Pos: TPoint):Boolean;
    function LegalMovesLeft(Player:TPlayerColor):Boolean;
    function KingsWillBeFacing(StartPos, EndPos: TPoint):Boolean;
    function PiecesInBetween(StartPos, EndPos: TPoint):Cardinal;
    function WillBeInCheck(StartPos, EndPos: TPoint; Player: TPlayerColor):Boolean;
    procedure Move(StartPos, EndPos: TPoint);
    function MoveIsLegal(StartPos, EndPos: TPoint):Boolean;
    procedure Reset;

    //coordinate conversion  move to public?
    class function XPosToBoard(XPos: Cardinal):Cardinal;
    class function YPosToBoard(YPos: Cardinal):Cardinal;
    class function PosToBoard(Pos: TPoint):TPoint;
    class function XBoardToPos(XBoard: Cardinal):Cardinal;
    class function YBoardToPos(YBoard: Cardinal):Cardinal;

    //property related
    function GetBoard(XPos, YPos: Cardinal):PPieceInfo;
    procedure SetBoard(XPos, YPos: Cardinal; Value: PPieceInfo);
    function GetLastMove:Cardinal;
    function GetPieceBitmap(PieceIndex: Cardinal):TBitmap;
    procedure SetPieceSet(Value: TPieceSet);
    function GetVersion:String;
  protected
    procedure DragCanceled;override;
    procedure DragDrop(Source: TObject; X, Y: Integer);override;
    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);override;
    procedure MouseDown(Button:TMouseButton; Shift:TShiftState;X,Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    //move related
    function DoMove(StartPos, EndPos: TPoint):Boolean;
    procedure PreviousMove;
    procedure NextMove;
    procedure GoToMove(MoveNumber: Cardinal);

    procedure DrawPiece(ACanvas: TCanvas; TopLeft: TPoint; Piece: PPieceInfo);
    procedure NewGame;
    class function OtherColor(Color: TPlayerColor):TPlayerColor;
  published
    property PieceBitmap[PieceIndex: Cardinal]:TBitmap read GetPieceBitmap;
    property Board[XPos, YPos: Cardinal]:PPieceInfo read GetBoard write SetBoard; default;
    property CurrentMove: Cardinal read FMove write GoToMove default 0;
    property LastMove: Cardinal read GetLastMove;
    property PieceSet: TPieceSet read FPieceSet write SetPieceSet default psCartoon;
    property Playing: Boolean read FPlaying write FPlaying default False;
    property ShowHint;
    property Turn: TPlayerColor read FTurn write FTurn default pcRed;
    property Version: String read GetVersion;
    property OnCheck: TOnCheckEvent read FOnCheck write FOnCheck default Nil;
    property OnMate: TOnMateEvent read FOnMate write FOnMate default Nil;
    property OnCapture: TOnCaptureEvent read FOnCapture write FOnCapture default Nil;
    property OnMove: TOnMoveEvent read FOnMove write FOnMove default Nil;
    property OnPreviousMove: TNotifyEvent read FOnPreviousMove write FOnPreviousMove default Nil;
    property OnNextMove: TNotifyEvent read FOnNextMove write FOnNextMove default Nil;
    property OnResurrect: TVoodooEvent read FOnResurrect write FOnResurrect default Nil;
    property OnReKill: TVoodooEvent read FOnReKill write FOnReKill default Nil;
    property OnPieceSetChange: TOnPieceSetChangeEvent
               read FOnPieceSetChange write FOnPieceSetChange default Nil;
  end;

//Utility routines
function Min(Num1,Num2:Cardinal):Cardinal;
function Max(Num1,Num2:Cardinal):Cardinal;
function Point(X, Y: Integer):TPoint;

procedure Register;

implementation

{$R bitmaps.res}

resourcestring
  XiangqiVersion = 'Xiangqi 1.0 Copyright Axel Fontaine 1998';

type
  TPieceArray = Array[0..Pred(MaxPieceBitmaps)] Of String;

const
  BoardRes = 'board';

  RedCastle: TRect = (TopLeft: (X: 3; Y: 0); BottomRight: (X: 5; Y: 2));
  GreenCastle: TRect = (TopLeft: (X: 3; Y: 7); BottomRight: (X: 5; Y: 9));

  RedPawn: TPieceInfo =
    (Color: pcRed; PieceType: ptPawn; ImageIndex: 0);
  RedCannon: TPieceInfo =
    (Color: pcRed; PieceType: ptCannon; ImageIndex: 2);
  RedElephant: TPieceInfo =
    (Color: pcRed; PieceType: ptElephant; ImageIndex: 4);
  RedKnight: TPieceInfo =
    (Color: pcRed; PieceType: ptKnight; ImageIndex: 6);
  RedRook: TPieceInfo =
    (Color: pcRed; PieceType: ptRook; ImageIndex: 8);
  RedGuard: TPieceInfo =
    (Color: pcRed; PieceType: ptGuard; ImageIndex: 10);
  RedKing: TPieceInfo =
    (Color: pcRed; PieceType: ptKing; ImageIndex: 12);
  GreenPawn: TPieceInfo =
    (Color: pcGreen; PieceType: ptPawn; ImageIndex: 1);
  GreenCannon: TPieceInfo =
    (Color: pcGreen; PieceType: ptCannon; ImageIndex: 3);
  GreenElephant: TPieceInfo =
    (Color: pcGreen; PieceType: ptElephant; ImageIndex: 5);
  GreenKnight: TPieceInfo =
    (Color: pcGreen; PieceType: ptKnight; ImageIndex: 7);
  GreenRook: TPieceInfo =
    (Color: pcGreen; PieceType: ptRook; ImageIndex: 9);
  GreenGuard: TPieceInfo =
    (Color: pcGreen; PieceType: ptGuard; ImageIndex: 11);
  GreenKing: TPieceInfo =
    (Color: pcGreen; PieceType: ptKing; ImageIndex: 13);

  InitialBoard : TGameBoard =
   ((Addr(RedRook),nil,nil,Addr(RedPawn),nil,
       nil,Addr(GreenPawn),nil,nil,Addr(GreenRook)),
    (Addr(RedKnight),nil,Addr(RedCannon),nil,nil,
       nil,nil,Addr(GreenCannon),nil,Addr(GreenKnight)),
    (Addr(RedElephant),nil,nil,Addr(RedPawn),nil,
       nil,Addr(GreenPawn),nil,nil,Addr(GreenElephant)),
    (Addr(RedGuard),nil,nil,nil,nil,
       nil,nil,nil,nil,Addr(GreenGuard)),
    (Addr(RedKing),nil,nil,Addr(RedPawn),nil,
       nil,Addr(GreenPawn),nil,nil,Addr(GreenKing)),
    (Addr(RedGuard),nil,nil,nil,nil,
       nil,nil,nil,nil,Addr(GreenGuard)),
    (Addr(RedElephant),nil,nil,Addr(RedPawn),nil,
       nil,Addr(GreenPawn),nil,nil,Addr(GreenElephant)),
    (Addr(RedKnight),nil,Addr(RedCannon),nil,nil,
       nil,nil,Addr(GreenCannon),nil,Addr(GreenKnight)),
    (Addr(RedRook),nil,nil,Addr(RedPawn),nil,
       nil,Addr(GreenPawn),nil,nil,Addr(GreenRook)));

  XiangqiPieceRes : TPieceArray =
    ('Pawn_r',            //  0
     'Pawn_g',            //  1
     'Cannon_r',          //  2
     'Cannon_g',          //  3
     'elephant_r',        //  4
     'elephant_g',        //  5
     'knight_r',          //  6
     'knight_g',          //  7
     'Rook_r',            //  8
     'Rook_g',            //  9
     'Guard_r',           // 10
     'Guard_g',           // 11
     'king_r',            // 12
     'king_g');           // 13 = Pred(MaxPieceBitmaps)

  XiangqiChPieceRes : TPieceArray =
    ('Pawn_r_ch',         //  0
     'Pawn_g_ch',         //  1
     'Cannon_r_ch',       //  2
     'Cannon_g_ch',       //  3
     'elephant_r_ch',     //  4
     'elephant_g_ch',     //  5
     'knight_r_ch',       //  6
     'knight_g_ch',       //  7
     'Rook_r_ch',         //  8
     'Rook_g_ch',         //  9
     'Guard_r_ch',        // 10
     'Guard_g_ch',        // 11
     'king_r_ch',         // 12
     'king_g_ch');        // 13 = Pred(MaxPieceBitmaps)

function Min(Num1,Num2:Cardinal):Cardinal;
begin
  If Num1 < Num2 Then Result := Num1 Else Result := Num2;
end;

function Max(Num1,Num2:Cardinal):Cardinal;
begin
  If Num1 > Num2 Then Result := Num1 Else Result := Num2;
end;

function Point(X, Y: Integer):TPoint;
begin
  Result.X := X;
  Result.Y := Y;
end;

//constructor - destructor
constructor TXiangqi.Create(AOwner: TComponent);
begin
  inherited;
  DragCursor := crHandPoint;
  Width := PieceWidth * MaxXPieces;
  Height := PieceHeight * MaxYPieces;
  FBoardBitmap := TBitmap.Create;
  FBoardBitmap.LoadFromResourceName(HInstance,BoardRes);
  FPieceBitmaps := TImageList.Create(Self);
  FPieceSet := psCartoon;
  FMoveList := TList.Create;
  FMove := 0;
  OrganizeBitmaps;
  Reset;
end;

destructor TXiangqi.Destroy;
var
  I: Integer;

begin
  For I := Pred(FMoveList.Count) DownTo 0 Do
    Begin
      Dispose(PMove(FMoveList[I]));
      FMoveList.Delete(I);
    End;
  FMoveList.Free;
  FPieceBitmaps.Free;
  FBoardBitmap.Free;
  inherited;
end;

//move related
function TXiangqi.DoMove(StartPos, EndPos: TPoint):Boolean;
begin
  Result := (FGameBoard[StartPos.X,StartPos.Y] <> Nil) And
            (FGameBoard[StartPos.X,StartPos.Y].Color = FTurn) And
            MoveIsLegal(StartPos,EndPos);
  Move(StartPos,EndPos);
end;

procedure TXiangqi.PreviousMove;
begin
  If (FMoveList.Count > 0) And (FMove > 0) Then
    Begin
      Dec(FMove);
      FGameBoard[PMove(FMoveList[FMove]).StartPos.X,PMove(FMoveList[FMove]).StartPos.Y] :=
        PMove(FMoveList[FMove]).Piece;
      FGameBoard[PMove(FMoveList[FMove]).EndPos.X,PMove(FMoveList[FMove]).EndPos.Y] :=
        PMove(FMoveList[FMove]).CapturedPiece;
      ClearBoardPos(PMove(FMoveList[FMove]).StartPos);
      ClearBoardPos(PMove(FMoveList[FMove]).EndPos);
      DrawPiece(Canvas,
                PosToBoard(PMove(FMoveList[FMove]).StartPos),
                PMove(FMoveList[FMove]).Piece);
      DrawPiece(Canvas,
                PosToBoard(PMove(FMoveList[FMove]).EndPos),
                PMove(FMoveList[FMove]).CapturedPiece);
      FTurn := OtherColor(FTurn);
      If (PMove(FMoveList[FMove]).CapturedPiece <> Nil) And Assigned(FOnResurrect) Then
        FOnResurrect(Self,PMove(FMoveList[FMove]).CapturedPiece);
      If Assigned(FOnPreviousMove) Then FOnPreviousMove(Self);
    End;
end;

procedure TXiangqi.NextMove;
begin
  If FMoveList.Count > FMove Then
    Begin
      FGameBoard[PMove(FMoveList[FMove]).StartPos.X,PMove(FMoveList[FMove]).StartPos.Y] :=
        Nil;
      FGameBoard[PMove(FMoveList[FMove]).EndPos.X,PMove(FMoveList[FMove]).EndPos.Y] :=
        PMove(FMoveList[FMove]).Piece;
      ClearBoardPos(PMove(FMoveList[FMove]).StartPos);
      ClearBoardPos(PMove(FMoveList[FMove]).EndPos);
      DrawPiece(Canvas,
                PosToBoard(PMove(FMoveList[FMove]).EndPos),
                PMove(FMoveList[FMove]).Piece);
      If (PMove(FMoveList[FMove]).CapturedPiece <> Nil) And Assigned(FOnReKill) Then
        FOnReKill(Self,PMove(FMoveList[FMove]).CapturedPiece);
      Inc(FMove);
      FTurn := OtherColor(FTurn);
      If Assigned(FOnNextMove) Then FOnNextMove(Self);
    End;
end;

procedure TXiangqi.GoToMove(MoveNumber: Cardinal);
begin
  If (MoveNumber <> CurrentMove) And (MoveNumber <= FMoveList.Count) Then
    If MoveNumber < CurrentMove Then
      Repeat
        PreviousMove;
      Until MoveNumber = CurrentMove
    Else   //cannot be equal so it must be at least 1 more than CurrentMove
      Repeat
        NextMove;
      Until MoveNumber = CurrentMove;
end;

//other public stuff
procedure TXiangqi.NewGame;
begin
  Reset;
  Paint;
end;

class function TXiangqi.OtherColor(Color: TPlayerColor):TPlayerColor;
begin
  If Color = pcRed Then Result := pcGreen
                   Else Result := pcRed;
end;

//Coordinate conversion routines
class function TXiangqi.XPosToBoard(XPos:Cardinal):Cardinal;
begin
  Result := XPos * PieceWidth;
end;

class function TXiangqi.YPosToBoard(YPos:Cardinal):Cardinal;
begin
  Result := YPos * PieceHeight;
end;

class function TXiangqi.PosToBoard(Pos: TPoint):TPoint;
begin
  Result.X := XPosToBoard(Pos.X);
  Result.Y := YPosToBoard(Pos.Y);
end;

class function TXiangqi.XBoardToPos(XBoard:Cardinal):Cardinal;
begin
  Result := XBoard div PieceWidth;
end;

class function TXiangqi.YBoardToPos(YBoard:Cardinal):Cardinal;
begin
  Result := YBoard div PieceHeight;
end;

//other
procedure TXiangqi.OrganizeBitmaps;
var
  I: Integer;
  TheImage: TBitmap;

begin
  FPieceBitmaps.Clear;
  FPieceBitmaps.Masked := True;
  FPieceBitmaps.DrawingStyle := dsTransparent;
  FPieceBitmaps.Width := PieceWidth;
  FPieceBitmaps.Height := PieceHeight;
  TheImage := TBitmap.Create;
  try
    For I := Low(TPieceArray) To High(TPieceArray) Do
      Begin
        TheImage.FreeImage;
        If FPieceSet = psCartoon Then
          TheImage.LoadFromResourceName(HInstance, XiangqiPieceRes[I])
        Else
          TheImage.LoadFromResourceName(HInstance, XiangqiChPieceRes[I]);
        FPieceBitmaps.Add(TheImage,TheImage);
      End;
  finally
    TheImage.Free;
  end;
end;

procedure TXiangqi.Reset;
var
  I: Integer;  //not cardinal !!!

begin
  For I := Pred(FMoveList.Count) DownTo 0 Do
    Begin
      Dispose(PMove(FMoveList[I]));
      FMoveList.Delete(I);
    End;
  FRedKingPos := Point(MaxXPieces Shr 1,0);
  FGreenKingPos := Point(MaxXPieces Shr 1,Pred(MaxYPieces));
  FTurn := pcRed;
  FGameBoard := InitialBoard;
end;

//Actual Xiangqi
function TXiangqi.HasCrossedRiver(Player: TPlayerColor; Pos: TPoint):Boolean;
begin
  Result := False;
  case Player of
    pcRed: If Pos.Y > Pred(MaxYPieces Shr 1) Then Result := True;
    pcGreen: If Pos.Y < (MaxYPieces Shr 1) Then Result := True;
  end;
end;

function TXiangqi.HasReachedEnd(Player: TPlayerColor; Pos: TPoint):Boolean;
begin
  Result := False;
  case Player of
    pcRed: If Pos.Y = Pred(MaxYPieces) Then Result := True;
    pcGreen: If Pos.Y = 0 Then Result := True;
  end;
end;

function TXiangqi.IsInCastle(Player: TPlayerColor; Pos: TPoint):Boolean;
var
  Castle: TRect;

begin
  case Player of
    pcRed: Castle := RedCastle;
    pcGreen: Castle := GreenCastle;
  end;
  If (Pos.X < Castle.Left) Or (Pos.X > Castle.Right) Or
     (Pos.Y < Castle.Top) Or (Pos.Y > Castle.Bottom) Then
    Result := False
  Else
    Result := True;
end;

function TXiangqi.KingsWillBeFacing(StartPos, EndPos: TPoint):Boolean;
begin
  Result := (StartPos.X <> EndPos.X) And
            (StartPos.X = FRedKingPos.X) And
            (StartPos.X = FGreenKingPos.X) And
            (PiecesInBetween(FRedKingPos,FGreenKingPos) = 1);
end;

function TXiangqi.PiecesInBetween(StartPos, EndPos: TPoint):Cardinal;
var
  I: Cardinal;

begin
  Result := 0;
  If StartPos.X = EndPos.X Then
    Begin
      For I := Succ(Min(StartPos.Y,EndPos.Y)) To Pred(Max(StartPos.Y,EndPos.Y)) Do
        If FGameBoard[StartPos.X,I] <> Nil Then Inc(Result);
    End
  Else
    Begin
      For I := Succ(Min(StartPos.X,EndPos.X)) To Pred(Max(StartPos.X,EndPos.X)) Do
        If FGameBoard[I,StartPos.Y] <> Nil Then Inc(Result);
    End;
end;

function TXiangqi.WillBeInCheck(StartPos, EndPos: TPoint; Player: TPlayerColor):Boolean;
var
  Temp: PPieceInfo;
  I, J: Cardinal;
  KingPos: TPoint;
  PawnYMove: Integer;

begin
  Result := False;
  Temp := FGameBoard[EndPos.X,EndPos.Y];
  FGameBoard[EndPos.X,EndPos.Y] := FGameBoard[StartPos.X,StartPos.Y];
  FGameBoard[StartPos.X,StartPos.Y] := Nil;
  If Player = pcRed Then
    Begin
      If (FGameBoard[EndPos.X,EndPos.Y] <> Nil) And
         (FGameBoard[EndPos.X,EndPos.Y].PieceType = ptKing) Then KingPos := EndPos
                                                            Else KingPos := FRedKingPos;
      PawnYMove := -1;
    End
  Else
    Begin
      If (FGameBoard[EndPos.X,EndPos.Y] <> Nil) And
         (FGameBoard[EndPos.X,EndPos.Y].PieceType = ptKing) Then KingPos := EndPos
                                                            Else KingPos := FGreenKingPos;
      PawnYMove := 1;
    End;

  I := 0;
  repeat
    J := 0;
    repeat
      If (FGameBoard[I,J] <> Nil) And (FGameBoard[I,J].Color <> Player) Then
        Case FGameBoard[I,J].PieceType Of
          ptPawn:
            If ((I = KingPos.X) And (J = (KingPos.Y-PawnYMove))) Or
               ((Abs(I-KingPos.X) = 1) And (J = KingPos.Y)) Then Result := True;

          ptCannon:
            If ((I = KingPos.X) Or (J = KingPos.Y)) And
               (PiecesInBetween(Point(I,J),KingPos) = 1) Then Result := True;

//        ptElephant: Cannot cross river => cannot put in check !!!

          ptKnight:
            If (((Abs(I-KingPos.X) = 1) And (Abs(J-KingPos.Y) = 2)) And
                (FGameBoard[I,(J+KingPos.Y) Shr 1] = Nil))
            Or
               (((Abs(I-KingPos.X) = 2) And (Abs(J-KingPos.Y) = 1)) And
                (FGameBoard[(I+KingPos.X) Shr 1,J] = Nil))
            Then
              Result := True;

          ptRook:
            If ((I = KingPos.X) Or (J = KingPos.Y)) And
               (PiecesInBetween(Point(I,J),KingPos) = 0) Then Result := True;

//        ptGuard: Cannot leave castle => cannot put in check !!!

//        ptKing: Cannot leave castle => cannot put in check !!!
        End;
      Inc(J);
    Until  (J = MaxYPieces) Or Result;
    Inc(I);
  Until (I = MaxXPieces) Or Result;

  FGameBoard[StartPos.X,StartPos.Y] := FGameBoard[EndPos.X,EndPos.Y];
  FGameBoard[EndPos.X,EndPos.Y] := Temp;
end;

function TXiangqi.MoveIsLegal(StartPos, EndPos: TPoint):Boolean;
var
  Src, Dest: PPieceInfo;

begin
  Result := False;
  If (StartPos.X >= 0) And (StartPos.X < MaxXPieces) And
     (StartPos.Y >= 0) And (StartPos.Y < MaxYPieces) And
     (EndPos.X >= 0) And (EndPos.X < MaxXPieces) And
     (EndPos.Y >= 0) And (EndPos.Y < MaxYPieces) Then
    Begin
      Src := FGameBoard[StartPos.X,StartPos.Y];
      Dest := FGameBoard[EndPos.X,EndPos.Y];
      If ((Dest <> Nil) And (Src.Color <> Dest.Color)) Or (Dest = Nil) Then
        case Src.PieceType of
          ptPawn:
            If (((((EndPos.Y-StartPos.Y) = 1) And (Src.Color = pcRed)) Or
                (((EndPos.Y-StartPos.Y) = -1) And (Src.Color = pcGreen))) And
               (Not HasReachedEnd(Src.Color,StartPos)) And (StartPos.X = EndPos.X))
            Or
               ((HasCrossedRiver(Src.Color,StartPos) And
               (Abs(StartPos.X-EndPos.X) = 1) And (StartPos.Y = EndPos.Y))
            And Not KingsWillBeFacing(StartPos,EndPos))
            Then
              Result := True;

          ptCannon:
            If (((Dest = Nil) And
                ((StartPos.X = EndPos.X) Or (StartPos.Y = EndPos.Y)) And
                (PiecesInBetween(StartPos,EndPos) = 0))
            Or
               ((Dest <> Nil) And
                ((StartPos.X = EndPos.X) Or (StartPos.Y = EndPos.Y)) And
                (PiecesInBetween(StartPos,EndPos) = 1)))
            And Not KingsWillBeFacing(StartPos,EndPos)
            Then
              Result := True;

          ptElephant:
            If ((Abs(StartPos.X-EndPos.X) = 2) And (Abs(StartPos.Y-EndPos.Y) = 2) And
               (FGameBoard[(StartPos.X+EndPos.X) Shr 1,(StartPos.Y+EndPos.Y) Shr 1] = Nil) And
               (Not HasCrossedRiver(Src.Color,EndPos)))
            And Not KingsWillBeFacing(StartPos,EndPos)
            Then
              Result := True;

          ptKnight:
            If ((((Abs(StartPos.X-EndPos.X) = 1) And (Abs(StartPos.Y-EndPos.Y) = 2)) And
                (FGameBoard[StartPos.X,(StartPos.Y+EndPos.Y) Shr 1] = Nil))
            Or
               (((Abs(StartPos.X-EndPos.X) = 2) And (Abs(StartPos.Y-EndPos.Y) = 1)) And
                (FGameBoard[(StartPos.X+EndPos.X) Shr 1,StartPos.Y] = Nil)))
            And Not KingsWillBeFacing(StartPos,EndPos)
            Then
              Result := True;

          ptRook:
            If (((StartPos.X = EndPos.X) Or (StartPos.Y = EndPos.Y)) And
               (PiecesInBetween(StartPos,EndPos) = 0))
            And Not KingsWillBeFacing(StartPos,EndPos)
            Then
              Result := True;

          ptGuard:
            If (IsInCastle(Src.Color,EndPos) And
               (Abs(StartPos.X-EndPos.X) = 1) And (Abs(StartPos.Y-EndPos.Y) = 1))
            And Not KingsWillBeFacing(StartPos,EndPos)
            Then
              Result := True;

          ptKing:
            If (IsInCastle(Src.Color,EndPos) And
               (((StartPos.X = EndPos.X) And (Abs(StartPos.Y-EndPos.Y) = 1)) Or
                ((StartPos.Y = EndPos.Y) And (Abs(StartPos.X-EndPos.X) = 1))))
            And Not
               (((Src.Color = pcRed) And (EndPos.X = FGreenKingPos.X) And
                 (PiecesInBetween(EndPos,FGreenKingPos) = 0)) Or
                ((Src.Color = pcGreen) And (EndPos.X = FRedKingPos.X) And
                 (PiecesInBetween(EndPos,FRedKingPos) = 0)))
            Then
              Result := True;
        end;
      If Result And WillBeInCheck(StartPos,EndPos,Src.Color) Then Result := False;
    End;
end;

function TXiangqi.LegalMovesLeft(Player: TPlayerColor):Boolean;
var
  I, J, K: Cardinal;
  PawnYMove: Integer;

begin
  Result := False;
  If Player = pcRed Then PawnYMove := -1 Else PawnYMove := 1;
  I := 0;
  repeat
    J := 0;
    repeat
      If (FGameBoard[I,J] <> Nil) And (FGameBoard[I,J].Color = Player) Then
        Case FGameBoard[I,J].PieceType Of
          ptPawn:
            If (MoveIsLegal(Point(I,J),Point(I,J+PawnYMove))) Or
               (MoveIsLegal(Point(I,J),Point(I-1,J))) Or
               (MoveIsLegal(Point(I,J),Point(I+1,J))) Then Result := True;

          ptCannon:
            Begin
              For K := 0 To Pred(MaxXPieces) Do
                If (I <> K) And MoveIsLegal(Point(I,J),Point(K,J)) Then Result := True;
              If Not Result Then
                For K := 0 To Pred(MaxYPieces) Do
                  If (J <> K) And MoveIsLegal(Point(I,J),Point(I,K)) Then Result := True;
            End;

          ptElephant:
            If (MoveIsLegal(Point(I,J),Point(I-2,J-2))) Or
               (MoveIsLegal(Point(I,J),Point(I-2,J+2))) Or
               (MoveIsLegal(Point(I,J),Point(I+2,J-2))) Or
               (MoveIsLegal(Point(I,J),Point(I+2,J+2))) Then Result := True;

          ptKnight:
            If (MoveIsLegal(Point(I,J),Point(I-2,J-1))) Or
               (MoveIsLegal(Point(I,J),Point(I-2,J+1))) Or
               (MoveIsLegal(Point(I,J),Point(I+2,J-1))) Or
               (MoveIsLegal(Point(I,J),Point(I+2,J+1))) Or
               (MoveIsLegal(Point(I,J),Point(I-1,J-2))) Or
               (MoveIsLegal(Point(I,J),Point(I-1,J+2))) Or
               (MoveIsLegal(Point(I,J),Point(I+1,J-2))) Or
               (MoveIsLegal(Point(I,J),Point(I+1,J+2))) Then Result := True;

          ptRook:
            Begin
              For K := 0 To Pred(MaxXPieces) Do
                If (I <> K) And MoveIsLegal(Point(I,J),Point(K,J)) Then Result := True;
              If Not Result Then
                For K := 0 To Pred(MaxYPieces) Do
                  If (J <> K) And MoveIsLegal(Point(I,J),Point(I,K)) Then Result := True;
            End;

          ptGuard:
            If (MoveIsLegal(Point(I,J),Point(I-1,J-1))) Or
               (MoveIsLegal(Point(I,J),Point(I-1,J+1))) Or
               (MoveIsLegal(Point(I,J),Point(I+1,J-1))) Or
               (MoveIsLegal(Point(I,J),Point(I+1,J+1))) Then Result := True;

          ptKing:
            If (MoveIsLegal(Point(I,J),Point(I,J-1))) Or
               (MoveIsLegal(Point(I,J),Point(I,J+1))) Or
               (MoveIsLegal(Point(I,J),Point(I-1,J))) Or
               (MoveIsLegal(Point(I,J),Point(I+1,J))) Then Result := True;
        End;
      Inc(J);
    Until  (J = MaxYPieces) Or Result;
    Inc(I);
  Until (I = MaxXPieces) Or Result;
end;

procedure TXiangqi.Move(StartPos, EndPos: TPoint);
var
  I: Cardinal;
  Move: PMove;

begin
  If FMove < FMoveList.Count Then
    For I := Pred(FMoveList.Count) DownTo FMove Do
      Begin
        Dispose(PMove(FMoveList[I]));
        FMoveList.Delete(I);
      End;
  New(Move);
  Move.StartPos := StartPos;
  Move.EndPos := EndPos;
  Move.Piece := FGameBoard[StartPos.X,StartPos.Y];
  Move.CapturedPiece := FGameBoard[EndPos.X,EndPos.Y];
  FMoveList.Add(Move);
  Inc(FMove);
  Exclude(FInCheck,FTurn); //If you are in check you cannot move unless you get out of check
  If WillBeInCheck(StartPos,EndPos,OtherColor(FTurn)) Then
    Include(FInCheck,OtherColor(FTurn))
  Else
    Exclude(FInCheck,OtherColor(FTurn));
  FGameBoard[EndPos.X,EndPos.Y] := FGameBoard[StartPos.X,StartPos.Y];
  FGameBoard[StartPos.X,StartPos.Y] := Nil;
  ClearBoardPos(StartPos);
  ClearBoardPos(EndPos);
  DrawPiece(Canvas,PosToBoard(EndPos),FGameBoard[EndPos.X,EndPos.Y]);
  If (FGameBoard[EndPos.X,EndPos.Y] <> Nil) And
     (FGameBoard[EndPos.X,EndPos.Y].PieceType = ptKing) Then
    If FTurn = pcRed Then FRedKingPos := EndPos Else FGreenKingPos := EndPos;
  //They can never be both in check at the same time
  If OtherColor(FTurn) In FInCheck Then
    If Assigned(FOnCheck) Then FOnCheck(Self,OtherColor(FTurn));
  If Move.CapturedPiece = Nil Then
    Begin
      If Assigned(FOnMove) Then FOnMove(Self,StartPos,EndPos);
    End
  Else
    Begin
      If Assigned(FOnCapture) Then FOnCapture(Self,StartPos,EndPos,Move.CapturedPiece);
    End;
  FTurn := OtherColor(FTurn);
  If Not LegalMovesLeft(FTurn) Then
    Begin
      FPlaying := False;
      If Assigned(FOnMate) Then
        If FTurn In FInCheck Then FOnMate(Self,FTurn,mtCheckMate)
                             Else FOnMate(Self,FTurn,mtStaleMate);
    End;
end;

//Property related routines
function TXiangqi.GetBoard(XPos, YPos: Cardinal):PPieceInfo;
begin
  If (XPos < MaxXPieces) And (YPos < MaxYPieces) Then
    Result := FGameBoard[XPos,YPos]
  Else
    Result := Nil;              //Error, so return nil
end;

procedure TXiangqi.SetBoard(XPos, YPos: Cardinal; Value: PPieceInfo);
begin
  If (XPos < MaxXPieces) And (YPos < MaxYPieces) Then
    FGameBoard[XPos,YPos] := Value;
end;

function TXiangqi.GetLastMove:Cardinal;
begin
  Result := FMoveList.Count;
end;

function TXiangqi.GetPieceBitmap(PieceIndex: Cardinal):TBitmap;
begin
  Result := Nil;
  If PieceIndex < MaxPieceBitmaps Then
    Begin
      FPieceBitmaps.GetBitmap(PieceIndex,Result);
      Result.Transparent := True;
      Result.TransParentColor := Result.Canvas.Pixels[0,0];
    End;
end;

procedure TXiangqi.SetPieceSet(Value: TPieceSet);
begin
  If Value <> FPieceSet Then
    Begin
      FPieceSet := Value;
      OrganizeBitmaps;
      Paint;
      If Assigned(FOnPieceSetChange) Then FOnPieceSetChange(Self, Value);
    End;
end;

function TXiangqi.GetVersion:string;
begin
  Result := XiangqiVersion;
end;

//Hint stuff
procedure TXiangqi.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  Hint := '';
  If FGameBoard[XBoardToPos(X),YBoardToPos(Y)] <> Nil Then
    Begin
      If FGameBoard[XBoardToPos(X),YBoardToPos(Y)].Color = pcRed Then
        Hint := 'Red '
      Else
        Hint := 'Green ';
      Case FGameBoard[XBoardToPos(X),YBoardToPos(Y)].PieceType Of
        ptPawn: Hint := Hint + 'pawn';
        ptCannon: Hint := Hint + 'cannon';
        ptElephant: Hint := Hint + 'elephant';
        ptKnight: Hint := Hint + 'knight';
        ptRook: Hint := Hint + 'rook';
        ptGuard: Hint := Hint + 'guard';
        ptKing: Hint := Hint + 'king';
      End;
    End;
end;

//Piece dragging
procedure TXiangqi.MouseDown(Button: TMouseButton;
                             Shift: TShiftState; X, Y: Integer);
var
  XPos, YPos: Cardinal;
begin
  inherited;
  If FPlaying Then
    Begin
      XPos := XBoardToPos(X);
      YPos := YBoardToPos(Y);
      If (FGameBoard[XPos,YPos] <> Nil) And//There must be a piece
         (FGameBoard[Xpos,YPos].Color = FTurn) Then  //from the right color
        Begin
          FSrcPosClick.X := Xpos;
          FSrcPosClick.Y := YPos;
          BeginDrag(False);
        End;
    End;
end;

procedure TXiangqi.DragOver(Source:TObject; X,Y: Integer;
                            State: TDragState; var Accept: Boolean);
var
  Pos: TPoint;
  mid: Integer;

begin
  inherited;
  Pos.X := XBoardToPos(X);
  Pos.Y := XBoardToPos(Y);
  mid := PieceWidth shr 1;
  If Not FPieceBitmaps.Dragging Then
    begin
      ClearBoardPos(FSrcPosClick);
      FPieceBitmaps.SetDragImage(FGameBoard[FSrcPosClick.X,FSrcPosClick.Y].ImageIndex, 0,0);
      FPieceBitmaps.DragLock(Parent.Handle,X+Self.Left-mid,Y+Self.Top-mid);
      FPieceBitmaps.BeginDrag(Parent.Handle,0,0);
    end;
  FPieceBitmaps.DragMove(X+Self.Left-mid,Y+Self.Top-mid);
  FPieceBitmaps.ShowDragImage;
  If (Source = Self) And
    (MoveIsLegal(FSrcPosClick,Pos)
     Or ((FSrcPosClick.X = Pos.X) And (FSrcPosClick.Y = Pos.Y))) Then
    Accept := True;
end;

procedure TXiangqi.DragDrop(Source: TObject; X, Y: Integer);
begin
  inherited;
  If FPieceBitmaps.Dragging Then
    Begin
      FPieceBitmaps.HideDragImage;
      FPieceBitmaps.EndDrag;
      FPieceBitmaps.DragUnlock;
    end;
  FDestPosClick.X := XBoardToPos(X);
  FDestPosClick.Y := XBoardToPos(Y);
  If (FSrcPosClick.X <> FDestPosClick.X) Or (FSrcPosClick.Y <> FDestPosClick.Y) Then
    Move(FSrcPosClick,FDestPosClick)
  Else
    DrawPiece(Canvas,PosToBoard(FSrcPosClick),FGameBoard[FSrcPosClick.X,FSrcPosClick.Y]);
end;

procedure TXiangqi.DragCanceled;
begin
  FPieceBitmaps.HideDragImage;
  FPieceBitmaps.EndDrag;
  FPieceBitmaps.DragUnlock;
  DrawPiece(Canvas,PosToBoard(FSrcPosClick),FGameBoard[FSrcPosClick.X,FSrcPosClick.Y]);
//  if @_illegalmove<>nil then _illegalmove(Self, SquareClick1);
end;

//Drawing routines
procedure TXiangqi.ClearBoardPos(Pos: TPoint);
var
  DrawRect: TRect;
begin
  DrawRect.Left := XposToBoard(Pos.X);
  DrawRect.Right := DrawRect.Left + PieceWidth;
  DrawRect.Top := YPosToBoard(Pos.Y);
  DrawRect.Bottom := DrawRect.Top + PieceHeight;
  Canvas.CopyRect(DrawRect,FBoardBitmap.Canvas,DrawRect);
end;

procedure TXiangqi.DrawPiece(ACanvas: TCanvas; TopLeft: TPoint; Piece: PPieceInfo);
begin
  If Piece <> Nil Then FPieceBitmaps.Draw(ACanvas,TopLeft.X,TopLeft.Y,Piece.ImageIndex);
end;

procedure TXiangqi.DrawPieces;
var
  I, J: Cardinal;

begin
  For I := 0 To Pred(MaxXPieces) Do
    For J := 0 To Pred(MaxYPieces) Do
      If FGameBoard[I,J] <> Nil Then
        DrawPiece(Canvas,PosToBoard(Point(I,J)),FGameBoard[I,J]);
end;

procedure TXiangqi.Paint;
begin
  Canvas.Draw(0,0,FBoardBitmap);
  DrawPieces;
end;




procedure Register;
begin
  RegisterComponents('Games', [TXiangqi]);
end;

end.
